/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: DeferredIconProvider.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.concurrent.atomic.AtomicBoolean;
import java.awt.Image;
import java.awt.image.BufferedImage;
import org.openide.util.RequestProcessor;
import it.tidalwave.util.IconProvider;
import it.tidalwave.util.IconProvider.Type;
import it.tidalwave.util.MutableIconProvider;
import it.tidalwave.util.NotFoundException;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: DeferredIconProvider.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public abstract class DeferredIconProvider extends MutableIconProvider
  {
    private static final Image EMPTY_ICON = new BufferedImage(16, 16, BufferedImage.TYPE_4BYTE_ABGR);

    @CheckForNull
    private Image icon;

    @CheckForNull
    private final IconProvider iconProviderDelegate;

    private final AtomicBoolean loaded = new AtomicBoolean(false);

    public DeferredIconProvider()
      {
        this.iconProviderDelegate = null;
      }

    public DeferredIconProvider (final @Nonnull IconProvider iconProviderDelegate)
      {
        this.iconProviderDelegate = iconProviderDelegate;
      }

    @Override @Nonnull
    public Image getIcon (final @Nonnull Type type, final @Nonnegative int size)
      {
        if (!loaded.getAndSet(true))
          {
            RequestProcessor.getDefault().post(new Runnable()
              {
                @Override
                public void run()
                  {
                    final Image oldIcon = icon;

                    try
                      {
                        icon = createIcon(type, size);
                      }
                    catch (NotFoundException e)
                      {
                        icon = getDefaultIcon(type, size);
                      }

                    if (oldIcon != icon)
                      {
                        fireIconChange(oldIcon, icon); // FIXME: should be in the EDT?
                      }
                  }
              });
          }

        return (icon != null) ? icon : getDefaultIcon(type, size);
      }

    @Nonnull
    protected abstract Image createIcon (final @Nonnull Type type, final @Nonnegative int size)
      throws NotFoundException;

    @Nonnull
    private Image getDefaultIcon (final @Nonnull Type type, final @Nonnegative int size)
      {
        return (iconProviderDelegate != null) ? iconProviderDelegate.getIcon(type, size) : EMPTY_ICON;
      }
  }
